import { useSubmissionStore, useFormStore } from '@vue-unistra/formbuilder'
import { computed } from 'vue'
import { LAYOUT_COMPONENTS } from '@/stores/template'
import type { InputField } from '#/template'
import type { Metadata } from '#/item'

export const useMetaSave = () => {
  // TODO: Check if this ts-ignore is still relevant
  // @ts-ignore wait for formbuilder typing
  const { getValue } = useSubmissionStore()
  const inputFields = computed<InputField[]>(() => {
    const store = useFormStore()
    return (Object.values(store.getFields) as InputField[])
        .filter(f => !LAYOUT_COMPONENTS.includes(f.component))
  })

  const getCleanValue = (name: string) => {
    const rawValue = getValue(name)
    return !Array.isArray(rawValue) && typeof rawValue === 'object'
        ? {isOption: true, value: rawValue.value}
        : {isOption: false, value: rawValue}
  }

  const storedMetadata = computed(() => {
    return inputFields.value.reduce((meta: Metadata[], field) => {
      const { id, name, repeatable = false, multiple=false } = field
      const {isOption, value} = getCleanValue(name as string)
      meta.push({
        id,
        isOption,
        multiple,
        name,
        repeatable,
        value,
      })
      return meta
    }, [])
  })
  return {
    getCleanValue,
    getValue,
    inputFields,
    storedMetadata,
  }
}
