import { computed } from 'vue'
import { useI18n } from 'vue-i18n'

import ReferentialComponent from '@/components/template/referential/FormBuilderComponent.vue'
import ReferentialSelect from '@/components/template/referential/LevelSelect.vue'
import ReferentialSettings from '@/components/template/referential/ComponentSettings.vue'

import type { CustomComponents } from '@vue-unistra/formbuilder'
import type { ControlsCustom } from '@vue-unistra/formbuilder/types/Controls'

export function useReferentialComponent() {

  const { t } = useI18n()

  const referentialComponents: CustomComponents = fieldName => {
    switch (fieldName) {
      case 'PountComponent':
        return ReferentialComponent
      case 'ReferentialSelect':
        return ReferentialSelect
    }
  }

  const referentialSettings: CustomComponents = fieldName => {
    switch (fieldName) {
      case 'PountTab':
        return ReferentialSettings
    }
  }

  const referentialControls = computed<ControlsCustom>(() => {
    return {
      PountComponent: {
        name: t('referential.field'),
        localeName: '',
        hasInternationalization: false,
        icon: 'mdi-family-tree',
        id: 'PountComponent',
        toAdd: { component: 'PountComponent' },
        settingsModalTabs: [
          {
            name: 'Pount',
            component: 'PountTab',
          },
          {
            name: 'API',
            component: 'ApiTab',
          },
          {
            name: 'Conditional',
            component: 'ConditionalTab',
          },
        ],
        usfSubtypes: [],
        usfType: 'custom'
      },
    }
  })

  return {
    referentialComponents,
    referentialControls,
    referentialSettings,
  }
}
