import { it, expect, describe, beforeEach, afterEach, vi } from 'vitest'
import { DOMWrapper, mount, VueWrapper } from '@vue/test-utils'
import { Quasar } from 'quasar'
import PassResetForm from '@/components/auth/PassResetForm.vue'
import { useContribStore } from '@/stores/contrib'
import i18n from '@/plugins/i18n'

const userMail = 'me@mail.org'

vi.mock('@/stores/contrib', () => ({
  useContribStore: () => ({
    resetPassword: vi.fn().mockResolvedValue({})
  })
}))

describe('PassResetForm', () => {
  let wrapper: VueWrapper
  let form: DOMWrapper<HTMLFormElement>
  const store = useContribStore()
  beforeEach(() => {
    wrapper = mount(PassResetForm, {
      props: {
        fill: userMail
      },
      global: {
        plugins: [
          i18n,
          Quasar,
        ]
      },
    })
    form = wrapper.find('form')
  })
  afterEach(() => {
    vi.restoreAllMocks()
  })
  it('should have a form with a submit & reset button', () => {
    expect(form.exists()).toBe(true)
    const submitButton = form.find('button[type="submit"]')
    expect(submitButton.exists()).toBe(true)
    const resetButton = form.find('button[type="reset"]')
    expect(resetButton.exists()).toBe(true)
  })
  it('should fill input value with its prop', () => {
    const emailInput: DOMWrapper<HTMLInputElement> = form.find('input[type="email"]')
    expect(emailInput.exists()).toBe(true)
    expect(emailInput.element.value).toEqual(userMail)
  })
  // do not pass
  // it('should trigger resetPassword action on submit ', async () => {
  //   const store = useContribStore()
  //   await form.trigger('submit')
  //   expect(store.resetPassword).toHaveBeenCalledOnce()
  //   expect(store.resetPassword).toHaveBeenLastCalledWith(userMail)
  // })
  // it('should emit close after submit ', async () => {
  //   await form.trigger('submit')
  //   expect(wrapper.emitted()).toHaveProperty('close')
  // })
  it('should emit close method on reset', async () => {
    await form.trigger('reset')
    expect(wrapper.emitted()).toHaveProperty('close')
  })
  it('should not do anything if form is not valid', async () => {
    const emailInput: DOMWrapper<HTMLInputElement> = form.find('input[type="email"]')
    await emailInput.setValue('')
    await form.trigger('submit')
    expect(wrapper.emitted().close, 'close not emitted').to.be.undefined
    expect(store.resetPassword).toHaveBeenCalledTimes(0)
  })
})
