import { it, expect, describe, beforeEach, afterEach } from 'vitest'
import { mount } from '@vue/test-utils'
import { Quasar } from 'quasar'
import i18n from '@/plugins/i18n'
import RichTextInput from '@/components/common/RichTextInput.vue'

import type { ValidationRule } from 'quasar'
import type { TestWrapper } from '../../types'

type PropsType = {
  value?: string;
  label: string;
  rules: ValidationRule[];
}


describe.skip('RichTextInput', () => {
  let wrapper: TestWrapper<{ stripText: string, toolbar: any }>
  function mountComponent(props: PropsType) {
    wrapper = mount(RichTextInput, {
      props,
      global: {
        plugins: [
          i18n,
          Quasar,
        ]
      },
    })
  }
  afterEach(() => {
    wrapper.unmount()
  })

  describe('computed', () => {
    describe('text', () => {
      beforeEach(()=> {
        mountComponent({value: 'text', label: 'label', rules: []})
      })
      it('should render a editable div', () => {
        expect(wrapper.exists()).toBe(true)
        expect(wrapper.find('div[contenteditable="true"]').exists()).toBe(true)
      })
      it('should be init with props', () => {
        expect(wrapper.find('div[contenteditable="true"]')).to.equal('text')
      })
      it('should emit input value if changed', () => {
        wrapper.vm.text = 'new text'
        expect(wrapper.emitted().input).to.have.lengthOf(1)
        expect(wrapper.emitted().input[0]).to.eql(['new text'])
      })
    })
    describe('stripText ', () => {
      it('should remove outermost p html tag', () => {
        mountComponent({value: '<p>text</p>', label: '', rules: []})
        expect(wrapper.vm.stripText).to.equal('text')
      })
      it('should let untagged text unchanged', () => {
        mountComponent({value: 'text', label: '', rules: []})
        expect(wrapper.vm.stripText).to.equal('text')
      })
      it('should let inner p tag unchanged', () => {
        mountComponent({value: '<p><p>text</p></p>', label: '', rules: []})
        expect(wrapper.vm.stripText).to.equal('<p>text</p>')
      })
      it('should emit empty string to reset text, if changed', () => {
        mountComponent({value: 'text', label: '', rules: []})
        wrapper.vm.stripText = ''
        expect(wrapper.emitted().input).to.have.lengthOf(1)
        expect(wrapper.emitted().input[0]).to.eql([''])
      })
    })
    describe('toolbar', () => {
      beforeEach(() => {
        mountComponent({value: '', label: '', rules: []})
      })
      it('should init color with current vuetify active theme', () => {
        expect(wrapper.vm.$vuetify.theme.isDark, 'initial theme').to.be.false
        expect(wrapper.vm.toolbar.color).to.equal('#F0F0F0')
      })
      it('should init color with current vuetify active theme', () => {
        wrapper.vm.$vuetify.theme.isDark = true
        expect(wrapper.vm.toolbar.color).to.equal('#252525')
      })
    })
  })
})
