import {describe, it, expect} from 'vitest'
import {
  getPaginatedResult,
  // thumbnailDelete,
  // thumbnailPost
} from '@/stores/helpers'

describe('store helpers', () => {
  describe('getPaginatedResult', () => {
    it('should return results key content of responseData as pageContent', () => {
      expect(
          getPaginatedResult({results: [{id: 1}, {id: 2}]}, 1, 4).pageContent
      ).toEqual([{id: 1}, {id: 2}])
    })
    describe('pageCount return', () => {
      it('should equal pageNumber if responseData nextPage is null', async () => {
        expect(
            getPaginatedResult(
              { nextPage: null, results: [] }, 17, 36
            ).pageCount
            , 'pageCount match given pageNumber'
        ).toEqual(17)
      })
      it('should equal to count / pageSize ratio with a non zero page size', async () => {
        expect(
            getPaginatedResult(
                { nextPage: 18, count: 23, results: [], }, 17, 11
            ).pageCount
            , 'pageCount match ratio count/pageSize'
        ).toEqual(3)
      })
      it('should equal to count / (result length) ratio with a zero page size', async () => {
        expect(
            getPaginatedResult(
              { nextPage: 18, count: 23, results: [{},{},{},{}], }, 17, 0
            ).pageCount
            , 'pageCount match ratio count/pageSize'
        ).toEqual(6)
      })
    })
  })
  // describe('generic actions', () => {
  //
  //   const rootApi = 'items'
  //   const axiosError = new Error('api error')
  //   const axiosStub = {}
  //   const stubMethods = ['post', 'delete']
  //   const commit = sinon.stub()
  //   const dispatch = sinon.stub().resolves()
  //   const context = {
  //     commit,
  //     dispatch,
  //   }
  //   beforeEach(() => {
  //     stubMethods.forEach(m => {
  //       axiosStub[m] = sinon.stub(authority, m).returns(Promise.resolve())
  //     })
  //   })
  //   afterEach(() => {
  //     stubMethods.forEach(m => {axiosStub[m].restore()})
  //     sinon.reset()
  //   })
  //   describe('thumbnailPost', () => {
  //     const axiosError = new Error('api error')
  //     const rootApi = 'items'
  //     const postAction = thumbnailPost(rootApi)
  //     const objectId = 'e5a3-67b5'
  //     const file = new File(['content'], 'fileName', {type: 'image/png'})
  //
  //     const thumbnailExemple = {
  //       s: 'http://s3.domain.fr/s.png',
  //       m: 'http://s3.domain.fr/m.png',
  //       l: 'http://s3.domain.fr/l.png',
  //     }
  //     const runAction = () => postAction(context, {objectId, file})
  //
  //     let actionResponse
  //
  //     beforeEach(() => {
  //       axiosStub.post.resolves({
  //         data: {thumbnails: thumbnailExemple}
  //       })
  //     })
  //     it('should call post api', () => {
  //       runAction()
  //       expect(axiosStub.post.calledOnce).to.be.true
  //     })
  //     describe('on success', () => {
  //       let url, payload
  //       beforeEach(async () => {
  //         actionResponse = await runAction()
  //         ;[url, payload] = axiosStub.post.getCall(0).args
  //       })
  //       it('should return 100', () => {
  //         expect(actionResponse).equals(100)
  //       })
  //       it('should prefix post url by rootApi', () => {
  //         expect(url.startsWith(rootApi))
  //       })
  //       it('should forge url with objectId', () => {
  //         expect(url.endsWith(`/${objectId}/thumbnail/`))
  //       })
  //       it('should post a FormData', () => {
  //         expect(payload).instanceOf(FormData)
  //       })
  //       it('should post a FormData with a file and a Content-type', () => {
  //         expect(payload.has('file'), 'key file check').to.be.true
  //         expect(payload.has('Content-Type'), 'key "Content-Type" check').to.be.true
  //       })
  //       it('should commit a thumbnails update', () => {
  //         expect(commit.calledOnce, 'one commit').to.be.true
  //         const [
  //             mutationName,
  //             commitData
  //         ] = commit.getCall(0).args
  //         expect(mutationName).to.equal('THUMBNAILS_UPDATE', 'mutation check')
  //         expect(commitData).to.eql({thumbnails: thumbnailExemple}, 'commit data check')
  //       })
  //       it('should dispatch info notification', () => {
  //         expect(dispatch.calledOnce, 'one dispatch').to.be.true
  //         const [
  //             actionName,
  //             message
  //         ] = dispatch.getCall(0).args
  //         expect(actionName).to.equal('dialog/displayInfo')
  //         expect(message).contains(file.name, 'file name displayed')
  //       })
  //     })
  //     describe('on failure', () => {
  //       beforeEach(async () => {
  //         axiosStub.post.rejects(axiosError)
  //         actionResponse = await runAction()
  //       })
  //       it('should return 0', () => {
  //         expect(actionResponse).equals(0)
  //       })
  //       it('should notify the failure', () => {
  //         expect(dispatch.calledOnce, 'one dispatch').to.be.true
  //         const [
  //           actionName,
  //           {error, message}
  //         ] = dispatch.getCall(0).args
  //         expect(actionName).to.equal('dialog/displayError')
  //         expect(error).to.equal(axiosError, 'error dispatched')
  //         expect(message).to.contain(file.name, 'file name displayed')
  //       })
  //     })
  //   })
  //   describe('thumbnailDelete', () => {
  //     const deleteAction = thumbnailDelete(rootApi)
  //     const objectId = 'e5a3-67b5'
  //     const runAction = () => deleteAction(context, objectId)
  //
  //     it('should call delete api', () => {
  //       runAction()
  //       expect(axiosStub.delete.calledOnce).to.be.true
  //     })
  //     it('should notify on failure', async () => {
  //       axiosStub.delete.rejects(axiosError)
  //       await runAction()
  //       expect(dispatch.calledOnce, 'one dispatch').to.be.true
  //       const [
  //         actionName,
  //         {error, message}
  //       ] = dispatch.getCall(0).args
  //       expect(actionName).to.equal('dialog/displayError')
  //       expect(error).to.equal(axiosError, 'error dispatched')
  //       expect(message).to.equal(i18n.t('file.delete.error'), 'specific message displayed')
  //     })
  //     describe('on success', () => {
  //       let url
  //       beforeEach(async () => {
  //         await runAction()
  //         url = axiosStub.delete.getCall(0).args[0]
  //       })
  //       it('should prefix post url by rootApi', () => {
  //         expect(url.startsWith(rootApi))
  //       })
  //       it('should forge url with objectId', () => {
  //         expect(url.endsWith(`/${objectId}/thumbnail/`))
  //       })
  //       it('should commit a thumbnails delete', () => {
  //         expect(commit.calledOnce, 'one commit').to.be.true
  //         const mutationName = commit.getCall(0).args[0]
  //         expect(mutationName).to.equal('THUMBNAILS_DELETE', 'mutation check')
  //       })
  //       it('should dispatch info notification', () => {
  //         expect(dispatch.calledOnce, 'one dispatch').to.be.true
  //         const [
  //             actionName,
  //             message
  //         ] = dispatch.getCall(0).args
  //         expect(actionName).to.equal('dialog/displayInfo')
  //         expect(message).to.equal(i18n.t('file.delete.success'), 'specific message displayed')
  //       })
  //     })
  //   })
  // })
})
