import { describe, it, expect, beforeEach, afterEach, vi } from 'vitest'
import { createPinia, setActivePinia } from 'pinia'

import { useReferentialStore } from '@/stores/referential'

import type {Node, Referential} from '#/referential'
import type { SpyInstance } from 'vitest'

const initDefinition = () => ({
  levels: [
    {
      id: 'dadcefe6-c5f4-4038-9b62-1c8316835775',
      name: 'niveau 1'
    },
    {
      id: 'cdb58bb4-bf24-42b0-a2dd-96af83349b24',
      name: 'niveau 2'
    },
    {
      id: 'a4504b31-a94c-4070-8311-4082da4f42e1',
      name: 'niveau 3'
    }
  ],
  roots: [
    'b0e54e52-c95d-435e-910c-d0190ef2df9c',
    '8f3973b5-ab1f-4ebc-8e94-71984d723966'
  ],
  nodes: {
    'b0e54e52-c95d-435e-910c-d0190ef2df9c': {
      id: 'b0e54e52-c95d-435e-910c-d0190ef2df9c',
      name: 'racine1',
      level: 0,
      children: [
        '4ef38584-63e9-4b60-a666-e9d0288b84c5',
        'bf874f02-d051-44f0-8f5e-5e0d93fa83a3',
        'fbce0804-6c81-43d1-bea8-f51ceae751e6'
      ],
      ancestors: []
    },
    '4ef38584-63e9-4b60-a666-e9d0288b84c5': {
      id: '4ef38584-63e9-4b60-a666-e9d0288b84c5',
      name: 'nœud11',
      level: 1,
      children: [
        '467e6bd7-671d-44f0-990b-72488a8725fd',
        'c6af0c20-3399-494c-90a4-dbeff04ace2d'
      ],
      ancestors: ['b0e54e52-c95d-435e-910c-d0190ef2df9c']
    },
    '467e6bd7-671d-44f0-990b-72488a8725fd': {
      id: '467e6bd7-671d-44f0-990b-72488a8725fd',
      name: 'nœud111',
      level: 2,
      ancestors: ['4ef38584-63e9-4b60-a666-e9d0288b84c5']
    },
    'c6af0c20-3399-494c-90a4-dbeff04ace2d': {
      id: 'c6af0c20-3399-494c-90a4-dbeff04ace2d',
      name: 'nœud112',
      level: 2,
      ancestors: ['4ef38584-63e9-4b60-a666-e9d0288b84c5']
    },
    'bf874f02-d051-44f0-8f5e-5e0d93fa83a3': {
      id: 'bf874f02-d051-44f0-8f5e-5e0d93fa83a3',
      name: 'nœud12',
      level: 1,
      ancestors: ['b0e54e52-c95d-435e-910c-d0190ef2df9c']
    },
    'fbce0804-6c81-43d1-bea8-f51ceae751e6': {
      id: 'fbce0804-6c81-43d1-bea8-f51ceae751e6',
      name: 'nœud101',
      level: 2,
      ancestors: ['b0e54e52-c95d-435e-910c-d0190ef2df9c']
    },
    '8f3973b5-ab1f-4ebc-8e94-71984d723966': {
      id: '8f3973b5-ab1f-4ebc-8e94-71984d723966',
      name: 'racine2',
      level: 0,
      children: [
        '7bf767f3-f834-4b86-baf8-e66e2dfdfac8',
        '63c2ff74-1c83-4fea-8e06-e1037e430c4d'
      ],
      ancestors: []
    },
    '7bf767f3-f834-4b86-baf8-e66e2dfdfac8': {
      id: '7bf767f3-f834-4b86-baf8-e66e2dfdfac8',
      name: 'nœud21',
      level: 1,
      children: ['c8fd76ae-32f4-466a-adb3-a04abd58912e'],
      ancestors: ['8f3973b5-ab1f-4ebc-8e94-71984d723966']
    },
    'c8fd76ae-32f4-466a-adb3-a04abd58912e': {
      id: 'c8fd76ae-32f4-466a-adb3-a04abd58912e',
      name: 'nœud211',
      level: 2,
      ancestors: ['7bf767f3-f834-4b86-baf8-e66e2dfdfac8']
    },
    '63c2ff74-1c83-4fea-8e06-e1037e430c4d': {
      id: '63c2ff74-1c83-4fea-8e06-e1037e430c4d',
      name: 'nœud201',
      level: 2,
      ancestors: ['8f3973b5-ab1f-4ebc-8e94-71984d723966']
    },
  },
})

const initReferential = (): Referential => ({
  id: 'a8b14a8e-2c44-4423-afcf-62dd70de2c9a',
  project: {id: 'de9f6ed4-eb2a-4b09-b48d-fae091e2cf17', name: 'test'},
  creator: {id: '56139eb4-5845-491e-8a2e-b74c519cf683', name: 'user test'},
  name: 'suppression test',
  description: 'jeu de test',
  scope: 4,
  version: null,
  versionLabel: null,
  currentVersion: true,
  source: 'user test',
  languages: {default: '', available: [], translations: {}},
  definition: initDefinition(),
  lists: [],
  createdAt: '2023-03-17T15:06:50.711241+01:00',
  updatedAt: '2023-03-17T15:06:50.711268+01:00',
  rootVersion: null,
})

vi.mock('@/services/alertService', () => ({
    notifyError: vi.fn(),
    notifySuccess: vi.fn()
  })
)

vi.mock('@/services/referentialService', () => ({
    getProjectReferentials: vi.fn(),
    getReferential: vi.fn(),
    patchReferential: vi.fn(),
    patchScope: vi.fn(),
    postReferential: vi.fn(),
  })
)

describe('store/referential', () => {
  const baseReferential = initReferential()

  const firstRootNode = baseReferential.definition.nodes[baseReferential.definition.roots[0]] as Required<Node>

  setActivePinia(createPinia())
  const store = useReferentialStore()

  beforeEach(() => {
    store.$reset()
    store.referential = initReferential()
  })
  afterEach(() => {
    vi.restoreAllMocks()
  })
  describe('actions', () => {
    describe('nodeAdd', () => {
      const nodeLevel = 2
      const nodeName = 'node name'
      let createSpy: SpyInstance<[string, number], Node>
      beforeEach(() => {
        createSpy = vi.spyOn(store, 'nodeCreate')
      })
      it('should create a new node', () => {
        store.nodeAdd(nodeName, nodeLevel)
        expect(createSpy).toHaveBeenCalledOnce()
      })
      it('should add node id to roots if no ancestor is provided', () => {
        store.nodeAdd(nodeName, nodeLevel)
        const nodeId = createSpy.mock.results[0].value.id
        const roots = [...store.referential.definition.roots]
        expect(roots.pop()).toEqual(nodeId)
      })
      it('should add node id as last children of its ancestor if provided', () => {
        const ancestorId = firstRootNode.id
        store.nodeAdd(nodeName, nodeLevel, ancestorId)
        const nodeId = createSpy.mock.results[0].value.id
        expect((store.nodes[ancestorId].children as string[]).pop()).toEqual(nodeId)
      })
    })
    describe('nodeCreate', () => {
      const nodeLevel = 2
      const nodeName = 'node name'
      let createdNode: Node
      beforeEach(() => {
        createdNode = store.nodeCreate(nodeName, nodeLevel)
      })
      it('should return a node with id', () => {
        expect(Object.keys(createdNode).includes('id')).toBe(true)
        expect(typeof createdNode.id).toBe('string')
        expect(createdNode.name).toEqual(nodeName)
        expect(createdNode.level).toEqual(nodeLevel)
      })
      it('should add this node to referential node list', () => {
        expect(store.referential.definition.nodes[createdNode.id]).toStrictEqual(createdNode)
      })
    })
    describe('nodeDelete', () => {
      it('should remove targeted node', () => {
        store.nodeDelete(firstRootNode.id)
        expect(store.nodes[firstRootNode.id]).toBeUndefined()
      })
      it('should remove node id from roots list if node is root', () => {
        store.nodeDelete(firstRootNode.id)
        expect(store.referential.definition.roots.includes(firstRootNode.id)).toBe(false)
      })
      it('should remove all its descendants', () => {
        const descendants: string[] = firstRootNode.children.reduce((desc, childId) => {
          const childNode = baseReferential.definition.nodes[childId]
          desc.push(childId)
          if (childNode.children?.length) {
            desc.push(...childNode.children)
          }
          return desc
        }, [] as string[])
        store.nodeDelete(firstRootNode.id)
        descendants.forEach((descendantId) => {
          expect(store.referential.definition.nodes[descendantId]).toBeUndefined()
        })
      })
      it('should not clean own reference in ancestors children, if review not asked', () => {
        const childNodeId = baseReferential.definition.nodes[firstRootNode.children[0]].id
        store.nodeDelete(childNodeId, false)
        expect((store.nodes[firstRootNode.id].children as string[]).includes(childNodeId)).toBe(true)
      })
    })
    describe('levelDelete', () => {
      describe('root level deletion', () => {
        const rootLevelId = baseReferential.definition.levels[0].id
        it('should put next level nodes as roots', () => {
          const expectedRoots = Object.values(baseReferential.definition.nodes)
              .filter(node => node.level === 1)
          expectedRoots.sort((a, b) => a.name === b.name ? 0 : (a.name < b.name ? -1 : 1))

          store.levelDelete(rootLevelId)
          expect(store.referential.definition.roots)
              .toEqual(expectedRoots.map(node => node.id))
        })
        it('should delete orphan node of deeper level', () => {
          const roots = store.referential.definition.roots
          const nextOrphans = Object.values(store.nodes)
              .filter(node => node.level === 2 && roots.includes((node.ancestors as string[])[0]))
              .map(node => node.id)

          store.levelDelete(rootLevelId)
          const remainIds = Object.keys(store.nodes)
          nextOrphans.forEach(nodeId => {
            expect(remainIds.includes(nodeId)).toBe(false)
          })
        })
        it('should decrease all nodes level', () => {
          store.levelDelete(rootLevelId)
          const roots = store.rootNodes
          Object.values(store.nodes).forEach(node => {
            expect(node.level).toEqual(roots.includes(node) ? 0 : 1)
          })
        })
      })
      describe('last level deletion', () => {
        const lastLevelId = baseReferential.definition.levels[2].id
        it('should delete last level nodes', () => {
          store.levelDelete(lastLevelId)
          Object.values(store.nodes).forEach(node => {
            expect(node.level).toBeLessThan(2)
          })
        })
        it('should remove children reference from ancestors', () => {
          const baseNodes: Record<string, Node> = baseReferential.definition.nodes

          store.levelDelete(lastLevelId)
          const nodes = Object.values(store.nodes)
          nodes.forEach(node => {
            if (node.level === 0) {
              expect(node.children)
                  .toHaveLength((baseNodes[node.id].children as string[]).length - 1)
            }
            else {
              if (node.children) {
                expect(node.children).toHaveLength(0)
              }
            }
          })

        })
        it('should keep level of each node', () => {
          const previousNodes: Record<string, Node> = baseReferential.definition.nodes
          store.levelDelete(lastLevelId)
          Object.values(store.nodes).forEach(node => {
            expect(node.level).toEqual(previousNodes[node.id].level)
          })
        })
      })
      describe('intermediate level', () => {
        const secondLevelId = baseReferential.definition.levels[1].id
        const secondLevelNodeIds = Object.values(baseReferential.definition.nodes)
            .reduce((ids: string[], node) => {
              if (node.level === 1) {
                ids.push(node.id)
              }
              return ids
            }, [])
        it('should remove nodes from this level', () => {
          store.levelDelete(secondLevelId)
          secondLevelNodeIds.forEach(id => {
            expect(store.nodes[id]).toBeUndefined()
          })
        })
        it('should keep level consistency', () => {
          store.levelDelete(secondLevelId)
          Object.values(store.nodes).forEach(node => {
            if (node.level > 0) {
              expect(node.level).toEqual(baseReferential.definition.nodes[node.id].level - 1)
            }
          })
        })
        it('should remove node id from ancestor children list', () => {
          store.levelDelete(secondLevelId)
          Object.values(store.nodes).forEach(node => {
            if (node.children) {
              const noSecondLevelChild = node.children.every(id => !secondLevelNodeIds.includes(id))
              expect(noSecondLevelChild).toBe(true)
            }
          })
        })
        it('should add grandchildren to node ancestors children', () => {
          store.levelDelete(secondLevelId)
          secondLevelNodeIds.forEach((id) => {
            const {ancestors, children} = baseReferential.definition.nodes[id]
            if (children) {
              for (const ancestor of (ancestors as string[]).map(ancestorId => store.nodes[ancestorId])) {
                expect(
                    children.every(childId => (ancestor.children as string[]).includes(childId))
                ).toBe(true)
              }
            }
          })
        })
      })
    })
  })

})
