import type { MediaFile } from '#/file'
import { RendererName } from '#/file'

export interface SetItem {
  /** rights for project manager */
  canEdit?: boolean;
  description: string;
  id: string;
  isPublic?: boolean;
  thumbnails: ThumbnailSource;
  title: string;
  updatedAt: string;
}
export interface SearchItemSet {
  id: string;
  title: string;
}

/** type depends on Item multiple and repeatable properties */
export type ItemValue = string | string[] | Array<string[]>

export interface Metadata {
  /** template field id */
  id: string;
  /** template field name */
  name: string;
  /** field value, values or list of values */
  value: ItemValue;
  /** template field allow multiple values */
  multiple?: boolean;
  /** its value is an option of template field, see MetaSaveMixin */
  isOption?: boolean;
  /** values will be dispatch into duplicated fields */
  repeatable?: boolean;
}
export interface SearchItemMetadata {
  creator: string[];
  publisher: string;
  publicationYear: number;
  resourceType: string;
  resourceTypeGeneral: string;
}


export interface FileViewer {
  /** name of component used to read the file */
  component: RendererName;
  /** id of the MediaFile to be read */
  fileId: string;
  /** control download button visibility */
  isDownloadable: boolean;
}

export interface ThumbnailSource {
  /** url to object's 48×48 thumbnail */
  s?: string;
  /** url to object's 64×64 thumbnail */
  m?: string;
  /** url to object's 128×128 thumbnail */
  l?: string;
  /** url to object's 512×512 thumbnail */
  embed?: string;
}

export interface ItemCreator {
  id: string;
  name: string;
  username: string;
  email: string;
}
export type SearchItemCreator = Omit<ItemCreator, 'username' | 'email'>

export interface ItemDetail {
  description: string;
  isPublic: boolean;
}

export interface NewItem extends ItemDetail {
  metadata: Metadata[];
  /** parent set id */
  setId?: string;
  /** 255 chars max */
  title: string;
}

interface ItemSet {
  id: string;
  title: string;
  templateId: string;
  projectId: string;
}

export interface PountItem extends NewItem{
  /** true if user is project manager or creator of this item */
  canEdit?: boolean;
  /** true if itself, its parents set and project are public */
  canBeShared?: boolean;
  creator: ItemCreator;
  id: string;
  mediaFiles: MediaFile[];
  /** - parent set details */
  set: ItemSet;
  /** - urls of thumbnails */
  thumbnails: ThumbnailSource;
  /** viewer parameters */
  viewer: FileViewer | null
  updatedAt: string;
}

type SearchItemHighlight = object

export interface SearchItem {
  id: string;
  title: string;
  description: string;
  metadata: SearchItemMetadata;
  creator: SearchItemCreator;
  isPublic: boolean;
  thumbnail?: string
  project: SearchItemCreator;
  set: SearchItemSet;
  score: number;
  highlight: SearchItemHighlight
}

export interface ItemStoreState {
  current: NewItem | PountItem;
  setItems: SetItem[];
  setItemsCount: number;
  pageCount: number;
}

export type DuplicationTarget = {setId: string, metadata: Metadata[]}
