import { AbstractField, FieldWithId, OldField, Option, StartField } from '@vue-unistra/formbuilder/types/Form'

export interface Hierarchy {
  name: string;
  id: string;
}

interface ReferentialField extends FieldWithId {
  referentialId: string;
  levelUsage: boolean[];
}

interface ReferentialStartField extends AbstractField {
  fields?: LevelField[];
  options?: Option[];
  referentialId: string;
  levelUsage: boolean[];
}

interface LevelField extends OldField {
  id: string;
  levelId: string;
  level?: Hierarchy;
}

interface LevelStartField extends StartField {
  levelId: string;
}

export interface LevelSelect extends Hierarchy {
  name: string;
  label: string;
  id: string;
  options: {value: string, label: string, ancestors?: string[]}[];
}

export interface NodeBase {
  id: string;
  label: string;
}

export interface Node {
  id: string;
  name: string;
  ancestors: string[];
  children?: string[];
  level: number;
}

export interface NodeOption {
  value: string;
  label: string;
  ancestors?: string[]
}

export type LanguageDefinition = {
  label: string;
  code: string;
}

export type Translation = Record<string, string>

export type ReferentialLanguages = {
  available: string[];
  default: string;
  translations: Record<string, Translation>;
}

export type ReferentialDefinition = {
  levels: Hierarchy[];
  roots: string[];
  nodes: Record<string, Node>
}

export interface NewReferential {
  name: string;
  description: string;
  projectId?: string;
  scope: number;
  languages: ReferentialLanguages;
  definition: ReferentialDefinition;
}

export interface Referential extends NewReferential {
  id: string;
  creator: {id: string; name: string};
  project: {id: string; name: string} | null;
  currentVersion: boolean;
  version: string | null;
  versionLabel: string | null;
  rootVersion: string | null;
  source: string;
  lists: object[];
  createdAt: string;
  updatedAt: string;
}

export type LevelOption = {value: string, label: string, ancestors?: string[]}

export interface TemplateReferential extends Referential {
  levelOptions: Record<string, NodeOption[]>
}


export interface ReferentialState {
  langIso: LanguageDefinition[];
  codeToLang: Record<string, LanguageDefinition>;
  referential: NewReferential | Referential;
  projectReferentials: ProjectReferential[];
  projectReferentialsCount: number;
  pageCount: number;
  templateReferentials: Record<string, TemplateReferential>
}

export interface ProjectReferential {
  id: string;
  name: string;
  description: string;
  scope: 1 | 2 | 4;
  isManager: boolean;
  isExternal: boolean;
  createdAt: string;
  updatedAt: string;
  project?: {
    id: string;
    name: string;
  }
}

export interface ReferentialFilters {
  search?: string;
  project?: string;
}
