import type { SetTemplate } from './template'
import type { ThumbnailSource } from './item'

export interface SetProject {
  id: string;
  name: string;
  isManager: boolean;
  isPublic: boolean;
}

export interface SetCreator {
  id: string;
  name: string;
  email: string;
}
export type SearchSetCreator = Omit<ItemCreator, 'email'>

export interface PountSet {
  canContribute: boolean; // allows to create items
  creator: SetCreator;
  id: string;
  description: string;
  /** visible to everyone or restricted to project members*/
  isPublic: boolean;
  numItems: boolean;
  numPublicItems: boolean;
  project: SetProject;
  publisher: string;
  template: SetTemplate;
  thumbnails: ThumbnailSource;
  title: string;
  updatedAt: string;
}

export interface ProjectSet {
  canContribute: boolean;
  description: string;
  id: string;
  isPublic: boolean;
  numItems: boolean;
  templateId: string;
  title: string;
  thumbnails: ThumbnailSource;
  updatedAt: string;
}

export interface BuildingSet extends Pick<PountSet, 'title' | 'description' | 'publisher' | 'isPublic'>{
  fromTemplateId: string; // id of cloned template at set creation
}

export interface SetPatch {
  title?: string;
  description?: string;
  publisher?: string;
  isPublic?: boolean;
}

export interface SearchSet {
  id: string;
  title: string;
  description;
  creator: SearchSetCreator;
  managers: string[];
  publisher: string;
  isPublic: boolean;
  itemsCount: number;
  publicItemsCount: number;
  thumbnail: string | null;
  publicationDate: string | null;
  project: SearchSetCreator;
  score: number;
  highlight: SearchSetHighlight;
}

export interface SetStoreState {
  current: PountSet;
  projectSets: ProjectSet[];
  projectSetsCount: number;
  pageCount: number;
}
