export type Theme = 'light' | 'dark'

export type Language = 'en' | 'fr'

interface PreferenceOption<T> {
  id: T;
  label: string;
  value: T;
  isActive?: boolean
}

export type LocaleOption = PreferenceOption<OctantLanguage>

export type ThemeOption = PreferenceOption<OctantTheme>

export interface Password {
  newPassword: string;
  newPassword2: string;
  oldPassword?: string;
}
interface InvitationData {
  url: string;
  projectId: string;
}

export interface ExternalInvitationData extends InvitationData {
  email: string;
}

export interface InternalInvitationData extends InvitationData {
  username: string;
}

export interface UserInvitation extends Password {
  invitationId: string; // invitation uuid
  projectId: string; // project uuid
  email: string; // uneditable user email
  firstName: string;
  lastName: string;
}

export interface UserInfo {
  bio: string;
  institution: string;
  researchLabs: UserStructure[];
  affiliations?: string[]; // internal official affectations
  otherAffiliations: string[]; // optional extra affectations
}

export interface UserStructure {
  id: number;
  label: string;
  acronym: string;
  longName: string;
  shortName: string;
}

export interface UserProfile {
  id: string;
  username: string;
  baseUsername: string;
  firstName: string;
  lastName: string;
  email: string;
  isProjectCreator: boolean;
  isExternal: boolean;
  isAdmin: boolean;
  info: UserInfo;
  settings: Object;
}

export interface UserStoreState {
  profile: UserProfile;
  isLogged: boolean;
}

